<?php

/**
 * @file
 * Contains \Drupal\walkscore\WalkScore.
 */

namespace Drupal\walkscore;

use Drupal;

/**
 * URL api walkscore.
 */
define('WALKSCORE_API_URL', 'http://api.walkscore.com/score');

/**
 * WalkScore class to retrieve text associated with walkscore number.
 */
class WalkScore {


  public function __construct() {
  }

  /**
   * Get walk score using API Key and location data (in array form).
   *
   * Returns a score between 0 - 100, or false if there is an error.
   */
  public static function getWalkScore($location) {

    $logger = Drupal::logger('walkscore');

    $address = [];
    $score = FALSE;

    if (isset($location['street'])) {
      $address[] = $location['street'];
    }

    if (isset($location['city'])) {
      $address[] = $location['city'];
    }

    if (isset($location['province'])) {
      $address[] = $location['province'];
    }

    $query = [
      'wsapikey' => Drupal::config('walkscore.settings')->get('api_key'),
      'address' => urlencode(implode(",", $address)),
      'lat' => $location['lat'],
      'lon' => $location['lon'],
      'format' => "json",
    ];

    $url = WALKSCORE_API_URL . "?" . http_build_query($query);

    $ch = curl_init($url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);

    // execute!
    $response = json_decode(curl_exec($ch), TRUE);
    $statuscode = $response['status'];

    if ($statuscode == 1) {
      $score = $response['walkscore'];
    }
    else {
      $logger->error(t("Error calculating Walk Score.") . WalkScore::getStatusCodeDescription($statuscode));
    }

    curl_close($ch);
    unset($ch);

    return $score;
  }

  /**
   * Map the API error codes.
   */
  public static function getStatusCodeDescription($code_error) {
    $responses = [
      '1' => t('Walk Score successfully returned.'),
      '2' => t('Score is being calculated and is not currently available.'),
      '30' => t('Invalid latitude/longitude.'),
      '31' => t('Walk Score API internal error.'),
      '40' => t('Your WSAPIKEY is invalid.'),
      '41' => t('Your daily API quota has been exceeded.'),
      '42' => t('Your IP address has been blocked.'),
    ];

    return isset($responses[$code_error]) ? $responses[$code_error] : t('Error :code. Code not found.', [":code" => $code_error]);
  }

  /**
   * Returns a user friendly description from the score.
   *
   * See http://www.walkscore.com/methodology.shtml for details.
   */
  public static function getScoreDescription($score) {
    $description = "";

    switch (TRUE) {
      case $score == 0:
        $description = t("No Data");
        break;

      case $score < 50:
        $description = t("Car-Dependent");
        break;

      case $score < 70:
        $description = t("Somewhat Walkable");
        break;

      case $score < 90:
        $description = t("Very Walkable");
        break;

      case $score <= 100:
        $description = t("Walker's Paradise");
        break;
    }

    return $description;
  }

  /**
   * Returns a user friendly description from the score.
   *
   * See http://www.walkscore.com/methodology.shtml for details.
   */
  public static function getScoreDescriptionDetail($score) {
    $description = "";

    switch (TRUE) {
      case $score == 0:
        $description = t("Walk Score Error");
        break;

      case $score < 25:
        $description = t("Almost all errands require a car.");
        break;

      case $score < 50:
        $description = t("Most errands require a car.");
        break;

      case $score < 70:
        $description = t("Some errands can be accomplished on foot.");
        break;

      case $score < 90:
        $description = t("Most errands can be accomplished on foot.");
        break;

      case $score <= 100:
        $description = t("Daily errands do not require a car.");
        break;
    }
    return $description;
  }

  /**
   * Print out a sample walkscore.
   */
  public static function printTest() {

    $location = [
      'street' => "8 hickory st w",
      'city' => 'waterloo',
      'province' => 'ontario',
      'lat' => '43.4780345',
      'lon' => '-80.525996',
      'format' => "json",
    ];

    $score = WalkScore::getWalkScore($location);

    $output = "<div>Test Location: ";
    $output .= $location['street'] . ", " . $location['city'] . ", " . $location['province'] . "</div>";
    $output .= "<div>Score: " . $score . "</div>";
    $output .= "<div>" . WalkScore::getScoreDescription($score) . "</div>";
    $output .= "<div>" . WalkScore::getScoreDescriptionDetail($score) . "</div>";

    return $output;
  }

}
